library(tidyverse)
library(dplyr)
library(caret)

#nutritional database from doi:10.1016/j.aca.2010.06.017


###########################nime muutmine kuhugi suht algusesse sisse tuua

dataset = read_delim('daSilva.csv',
                     delim = ";",
                     locale=locale(decimal_mark = ","),
                     col_names = TRUE,
                     trim_ws = TRUE)


dataset <- dataset %>%
        select(-c(NDB_No, FdGrp_Cd, NDB_No_1, GmWt_Desc2, GmWt_Desc1)) %>%
        mutate(fruit = `00000Shrt_Desc`)

missing_values_row <- dataset %>%
        mutate(missing = rowSums(is.na(dataset))) %>%
        select(fruit, missing)

missing_values_column <- dataset %>%
        select(everything()) %>%  # replace to your needs
        summarise_all(funs(sum(is.na(.))))


dataset <- dataset%>%
        select_if(~ sum(is.na(.)) < 10) %>%
        drop_na()

dataset2 <- dataset %>% 
        select(-"00000Shrt_Desc") 

dataset2 <- dataset2%>%
        select(-nearZeroVar(dataset2)) 

dataset <- dataset2 %>%
        select(-findCorrelation(cor(dataset2), cutoff = 0.6))%>%
        mutate('00000Shrt_Desc' = dataset$`00000Shrt_Desc`) %>%
        select('00000Shrt_Desc', everything())

# Feature Scaling
dataset[,2:12] = scale(dataset %>% select(-c("00000Shrt_Desc")))

# #or replace misisng values with the average of the descriptor
# dataset <- dataset %>%
#         mutate(Fiber_TD = ifelse(is.na(Fiber_TD), mean(Fiber_TD, na.rm = TRUE), Fiber_TD)) #needs to be repeated with all varaibles having missing values
# 
# summary(dataset)

#------------------------------------------K-means Clustering------------------------------------------
# Feature Scaling is not needed in k-mean clustering
# Using the elbow method to find the optimal number of clusters
set.seed(6)
wcss = vector()
for (i in 1:50) wcss[i] = sum(kmeans(dataset %>% select(-c("00000Shrt_Desc")), i)$withinss)
plot(1:50,
     wcss,
     type = 'b',
     main = paste('The Elbow Method'),
     xlab = 'Number of clusters',
     ylab = 'WCSS')


# Fitting K-Means to the dataset
kmeans_14 = kmeans(x = dataset %>% select(-c("00000Shrt_Desc")), centers = 7, nstart = 10)
y_kmeans = kmeans_14$cluster

dataset <- dataset %>%
        mutate(clusters_assigned_kmeans = y_kmeans) %>%
        select("00000Shrt_Desc", clusters_assigned_kmeans, everything())


# Visualising the clusters
library(cluster)
clusplot(dataset[3:13],
         y_kmeans,
         lines = 0,
         shade = TRUE,
         color = TRUE,
         labels = 2,
         plotchar = FALSE,
         span = TRUE,
         main = paste('Clusters of commodity groups'),
         xlab = 'PC1',
         ylab = 'PC2')



#--------------------------------------hierarchical clustering----------------------------------------------



# Using the dendrogram to find the optimal number of clusters
dendrogram = hclust(d = dist(dataset %>% select(-c("00000Shrt_Desc", clusters_assigned_kmeans)), 
                             method = 'euclidean'), 
                    method = 'ward.D')
plot(dendrogram,
     main = paste('Dendrogram'),
     xlab = 'Customers',
     ylab = 'Euclidean distances')

# Fitting Hierarchical Clustering to the dataset
y_hc = cutree(dendrogram, 7)

dataset <- dataset %>%
        mutate(clusters_assigned_hc = y_hc) %>%
        select("00000Shrt_Desc", clusters_assigned_kmeans, clusters_assigned_hc, everything())

# Visualising the clusters
library(cluster)
clusplot(dataset[,4:14],
         y_hc,
         lines = 0,
         shade = TRUE,
         color = TRUE,
         labels= 2,
         plotchar = FALSE,
         span = TRUE,
         main = paste('Clusters of commodity groups'),
         xlab = 'PC1',
         ylab = 'PC2')

write_delim(dataset, "clutering_daSilva.csv", delim = ",")
